@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -11,13 +11,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -74,14 +74,14 @@
   #include <Inventor/nodes/SoBaseColor.h>
   #include <Inventor/nodes/SoCone.h>
   #include <Inventor/nodes/SoSeparator.h>
-  
+
   int
   main(int argc, char ** argv)
   {
     // Initializes SoQt library (and implicitly also the Coin and Qt
     // libraries). Returns a top-level / shell Qt window to use.
     QWidget * mainwin = SoQt::init(argc, argv, argv[0]);
-  
+
     // Make a dead simple scene graph by using the Coin library, only
     // containing a single yellow cone under the scenegraph root.
     SoSeparator * root = new SoSeparator;
@@ -92,12 +92,12 @@
     root->addChild(col);
 
     root->addChild(new SoCone);
-  
+
     // Use one of the convenient SoQt viewer classes.
     SoQtExaminerViewer * eviewer = new SoQtExaminerViewer(mainwin);
     eviewer->setSceneGraph(root);
     eviewer->show();
-  
+
     // Pop up the main window.
     SoQt::show(mainwin);
     // Loop until exit.
@@ -211,7 +211,7 @@
   For those who are using the implementations of the Inventor API from
   either SGI or TGS, we would like to point out that %SoQt can also be
   used on top of either of those libraries instead of the \COIN
-  library from Systems in Motion.
+  library from Kongsberg Oil & Gas Technologies.
 
   The %SoQt API is based on and closely matches the InventorXt library
   API, originally developed by SGI. This should make it
@@ -229,6 +229,11 @@
 #endif // HAVE_CONFIG_H
 #include <qt-config.h>
 
+//Due to the niceties of the preprocessor, and X11's exquisite use
+//thereof. Make sure this is included before any X11 includes, to
+//avoid seemingly ludicrous error messages.
+#include <Inventor/Qt/SoQtInternal.h>
+
 #include <stdlib.h>
 #include <limits.h>
 
@@ -238,6 +243,8 @@
 #include <qevent.h>
 #include <qapplication.h>
 #include <qmetaobject.h>
+#include <Inventor/C/threads/thread.h>
+
 
 #ifdef Q_WS_X11
 #include <X11/Xlib.h>
@@ -270,11 +277,10 @@
 #include <Inventor/Qt/devices/spwinput_win32.h>
 #endif // HAVE_WIN32_API
 
-
 #include <Inventor/Qt/SoQtComponent.h>
+#include <Inventor/Qt/SoQtSignalThread.h>
 #include <Inventor/Qt/SoAny.h>
-#include <Inventor/Qt/SoQtInternal.h>
-
+#include <Inventor/Qt/SoQtImageReader.h>
 #include <soqtdefs.h>
 
 // *************************************************************************
@@ -291,6 +297,7 @@ QTimer * SoQtP::timerqueuetimer = NULL;
 QTimer * SoQtP::delaytimeouttimer = NULL;
 SoQtP * SoQtP::slotobj = NULL;
 bool SoQtP::didcreatemainwidget = FALSE;
+SoQtImageReader * SoQtP::imagereader = NULL;
 
 // *************************************************************************
 
@@ -331,6 +338,11 @@ SoQtP_XErrorHandler * SoQtP::previous_handler = NULL;
 
 int SoQtP::DEBUG_LISTMODULES = ENVVAR_NOT_INITED;
 
+// these two are used to make SoQt thread safe (changing a scene graph
+// from a separate thread won't make Qt go haywire).
+unsigned long SoQtP::original_thread;
+SoQtSignalThread * SoQtP::signalthread = NULL;
+
 // *************************************************************************
 
 // We're using the singleton pattern to create a single SoQtP object
@@ -340,7 +352,24 @@ int SoQtP::DEBUG_LISTMODULES = ENVVAR_NOT_INITED;
 SoQtP *
 SoQtP::soqt_instance(void)
 {
-  if (!SoQtP::slotobj) { SoQtP::slotobj = new SoQtP; }
+  if (!SoQtP::slotobj) {
+    SoQtP::slotobj = new SoQtP;
+#if defined(__COIN__) && defined(SOQT_SIGNAL_THREAD_ACTIVE)
+    // store the thread SoQt::init() is called from, and start a signal thread to
+    // handle scene graph changes from other threads
+    SoQtP::original_thread = cc_thread_id();
+    SoQtP::signalthread = new SoQtSignalThread();
+    QObject::connect(SoQtP::signalthread, SIGNAL(triggerSignal()),
+                     SoQtP::slotobj, SLOT(slot_sensorQueueChanged()));
+    SoQtP::signalthread->start();
+#endif // __COIN__
+  }
+
+#if HAVE_SBIMAGE_ADDREADIMAGECB
+  if (!SoQtP::imagereader) {
+    SoQtP::imagereader = new SoQtImageReader();
+  }
+#endif // HAVE_SBIMAGE_ADDREADIMAGECB
   return SoQtP::slotobj;
 }
 
@@ -449,7 +478,7 @@ SoQtP::slot_idleSensor()
                            SoQtP::appobject->hasPendingEvents());
     SoDebugError::postInfo("SoQt::idleSensor", "is %s",
                            SoQtP::idletimer->isActive() ? "active" : "inactive");
-  } 
+  }
 #endif // HAVE_QAPPLICATION_HASPENDINGEVENTS
 
 #ifdef HAVE_QAPPLICATION_HASPENDINGEVENTS
@@ -519,8 +548,21 @@ SoQtP::slot_delaytimeoutSensor()
 // This function gets called whenever something has happened to any of
 // the sensor queues. It starts or reschedules a timer which will
 // trigger when a sensor is ripe for plucking.
+
 void
 SoGuiP::sensorQueueChanged(void *)
+{
+#if defined(__COIN__) && defined(SOQT_SIGNAL_THREAD_ACTIVE)
+  if (SoQtP::signalthread->isRunning() && (cc_thread_id() != SoQtP::original_thread)) {
+    SoQtP::signalthread->trigger();
+    return;
+  }
+#endif // __COIN__ && SOQT_SIGNAL_THREAD_ACTIVE
+  SoQtP::soqt_instance()->slot_sensorQueueChanged();
+}
+
+void
+SoQtP::slot_sensorQueueChanged(void)
 {
   // We need three different mechanisms to interface Coin sensor
   // handling with Qt event handling, which are:
@@ -542,9 +584,15 @@ SoGuiP::sensorQueueChanged(void *)
 
   if (!SoQtP::timerqueuetimer) {
     SoQtP::timerqueuetimer = new QTimer;
+#if QT_VERSION >= 0x040000
+    SoQtP::timerqueuetimer->setSingleShot(TRUE);
+#endif
     QObject::connect(SoQtP::timerqueuetimer, SIGNAL(timeout()),
                      SoQtP::soqt_instance(), SLOT(slot_timedOutSensor()));
     SoQtP::idletimer = new QTimer;
+#if QT_VERSION >= 0x040000
+    SoQtP::idletimer->setSingleShot(TRUE);
+#endif
     QObject::connect(SoQtP::idletimer, SIGNAL(timeout()),
                      SoQtP::soqt_instance(), SLOT(slot_idleSensor()));
     SoQtP::delaytimeouttimer = new QTimer;
@@ -576,11 +624,20 @@ SoGuiP::sensorQueueChanged(void *)
     }
 
     // Change interval of timerqueuetimer when head node of the
-    // timer-sensor queue of SoSensorManager changes. 
+    // timer-sensor queue of SoSensorManager changes.
     if (!SoQtP::timerqueuetimer->isActive())
-      SoQtP::timerqueuetimer->start((int)interval.getMsecValue(), TRUE);
-    else
+      SoQtP::timerqueuetimer->start((int)interval.getMsecValue()
+#if QT_VERSION < 0x040000
+				    , TRUE
+#endif
+	);
+    else {
+#if QT_VERSION >= 0x040000
+      SoQtP::timerqueuetimer->setInterval((int)interval.getMsecValue());
+#else
       SoQtP::timerqueuetimer->changeInterval((int)interval.getMsecValue());
+#endif
+    }
   }
   // Stop timerqueuetimer if queue is completely empty.
   else if (SoQtP::timerqueuetimer->isActive()) {
@@ -596,16 +653,24 @@ SoGuiP::sensorQueueChanged(void *)
                              "delaysensor pending");
     }
 
-    // Start idletimer at 0 seconds in the future. -- That means it will 
-    // trigger when the Qt event queue has been run through, i.e. when 
+    // Start idletimer at 0 seconds in the future. -- That means it will
+    // trigger when the Qt event queue has been run through, i.e. when
     // the application is idle.
-    if (!SoQtP::idletimer->isActive()) SoQtP::idletimer->start(0, TRUE);
+    if (!SoQtP::idletimer->isActive()) SoQtP::idletimer->start(0
+#if QT_VERSION < 0x040000
+							       , TRUE
+#endif
+      );
 
     if (!SoQtP::delaytimeouttimer->isActive()) {
       const SbTime & t = SoDB::getDelaySensorTimeout();
       if (t != SbTime::zero()) {
         unsigned long timeout = t.getMsecValue();
-        SoQtP::delaytimeouttimer->start((int)timeout, TRUE);
+        SoQtP::delaytimeouttimer->start((int)timeout
+#if QT_VERSION < 0x040000
+					, TRUE
+#endif
+	  );
       }
     }
   }
@@ -617,77 +682,6 @@ SoGuiP::sensorQueueChanged(void *)
 
 // *************************************************************************
 
-// We overload the QApplication class to be able to get hold of the
-// X11 events directly. (This is necessary to handle Spacetec
-// spaceball devices.)
-class SoQtApplication : public QApplication {
-public:
-  SoQtApplication(int argc, char ** argv) : QApplication(argc, argv) { 
-#ifdef HAVE_X11_AVAILABLE
-    this->display = NULL;
-#endif // HAVE_X11_AVAILABLE
-  }
-#ifdef HAVE_X11_AVAILABLE
-  virtual bool x11EventFilter(XEvent * e) {
-    SPW_InputEvent sbEvent;
-    if (SPW_TranslateEventX11(this->getDisplay(), e, &sbEvent)) {
-      QWidget * focus = this->focusWidget();
-      if (!focus) focus = this->activeWindow();
-      if (focus) {
-        QCustomEvent qevent((QEvent::Type)SoQtInternal::SPACEBALL_EVENT,
-                            (void *)&sbEvent);
-        QApplication::sendEvent(focus, &qevent);
-      }
-    }
-    return QApplication::x11EventFilter(e);
-  }
-  Display * getDisplay(void) {
-    QWidget * topw = SoQt::getTopLevelWidget();
-    if (topw) return topw->x11Display();
-    if (this->display == NULL) {
-      // Keep a single display-ptr.
-      // 
-      // This resource is never deallocated explicitly (but of course
-      // implicitly by the system on application close-down). This to
-      // work around some strange problems with the NVidia-driver 29.60
-      // on XFree86 v4 when using XCloseDisplay() -- like doublebuffered
-      // visuals not working correctly.
-      //
-      // Also, XCloseDisplay() has been known to cause crashes when
-      // running remotely from some old Mesa version on Red Hat Linux
-      // 6.2 onto an IRIX6.5 display. It seems likely that this was
-      // caused by a bug in that particular old Mesa version.
-      //
-      // mortene@sim.no
-      this->display = XOpenDisplay(NULL);
-    }
-    return this->display;
-  }
-  Display * display;
-#endif // HAVE_X11_AVAILABLE
-
-#ifdef HAVE_WIN32_API
-  virtual bool winEventFilter(MSG * msg) {
-    SPW_InputEvent sbEvent;
-    if (SPW_TranslateEventWin32(msg, &sbEvent)) {
-      QWidget * focus = this->focusWidget();
-      if (!focus) focus = this->activeWindow();
-      if (focus) {
-        QCustomEvent qevent((QEvent::Type)SoQtInternal::SPACEBALL_EVENT,
-                            (void *)&sbEvent);
-        QApplication::sendEvent(focus, &qevent);
-      }
-    }
-#if (QT_VERSION >= 0x040000)
-    long result = 0;
-    return QApplication::winEventFilter(msg, &result);
-#else
-    return QApplication::winEventFilter(msg);
-#endif
-  }
-#endif // HAVE_WIN32_API
-};
-
 #endif // DOXYGEN_SKIP_THIS
 
 // *************************************************************************
@@ -707,6 +701,9 @@ COIN_IV_EXTENSIONS
 void
 SoQt::init(QWidget * toplevelwidget)
 {
+  // make sure the SoQtP slotinstance is initialized in the main thread
+  SoQtP::soqt_instance();
+
   // workaround for Qt bug under Windows. See above for comments
   soqt_reset_simple_timers();
 
@@ -717,9 +714,11 @@ SoQt::init(QWidget * toplevelwidget)
     // subclass to catch spaceball events.
 
     // use a static char array to store the dummy argv parameters
-    static char * dummyargv[1];
+    static const char * dummyargv[1];
     dummyargv[0] = "SoQt";
-    SoQtP::appobject = new SoQtApplication(1, (char **) dummyargv);
+    int argc = 1;
+    SoQtP::appobject = new QApplication(argc, (char **) dummyargv);
+    //new SoQtApplication(argc, (char **) dummyargv);
     SoQtP::madeappobject = TRUE;
   }
   else {
@@ -767,7 +766,7 @@ SoQt::init(QWidget * toplevelwidget)
     const char * env = SoAny::si()->getenv(SoQtP::SOQT_X11_ERRORHANDLER);
     SoQtP::X11_ERRORHANDLER = env ? atoi(env) : 0;
   }
-      
+
   if (SoQtP::X11_ERRORHANDLER) {
     // Intervene upon X11 errors.
     SoQtP::previous_handler = (SoQtP_XErrorHandler*)XSetErrorHandler((XErrorHandler)SoQtP::X11Errorhandler);
@@ -797,7 +796,7 @@ SoQt::init(QWidget * toplevelwidget)
     SoQtP::DEBUG_LISTMODULES = env ? atoi(env) : 0;
     if (SoQtP::DEBUG_LISTMODULES) { SoAny::listWin32ProcessModules(); }
   }
-  
+
   SoDB::getSensorManager()->setChangedCallback(SoGuiP::sensorQueueChanged, NULL);
   SoQtP::mainwidget = toplevelwidget;
 }
@@ -806,6 +805,8 @@ SoQt::init(QWidget * toplevelwidget)
 QWidget *
 SoQt::init(int & argc, char ** argv, const char * appname, const char * classname)
 {
+  // make sure the SoQtP instance is initialized in the main thread
+  SoQtP::soqt_instance();
   // Must do this here so SoDebugError is initialized before it could
   // be attempted used.
   if (!SoDB::isInitialized()) { SoDB::init(); }
@@ -826,7 +827,7 @@ SoQt::init(int & argc, char ** argv, const char * appname, const char * classnam
   if (qApp == NULL) {
     // Set up the QApplication instance which we have derived into a
     // subclass to catch spaceball events.
-    SoQtP::appobject = new SoQtApplication(argc, argv);
+    SoQtP::appobject = new QApplication(argc, argv);
     SoQtP::madeappobject = TRUE;
   }
   else {
@@ -837,12 +838,22 @@ SoQt::init(int & argc, char ** argv, const char * appname, const char * classnam
     SoQtP::appobject = qApp;
   }
 
+#if QT_VERSION >= 0x040000
+  QWidget * mainw = new QWidget();
+  mainw->setObjectName(classname);
+#else
   QWidget * mainw = new QWidget(NULL, classname);
+#endif
   SoQtP::didcreatemainwidget = TRUE;
   SoQt::init(mainw);
 
+#if QT_VERSION >= 0x040000
+  if (appname) { SoQtP::mainwidget->setWindowTitle(appname); }
+#else
   if (appname) { SoQtP::mainwidget->setCaption(appname); }
   SoQtP::appobject->setMainWidget(SoQtP::mainwidget);
+#endif
+
   return SoQtP::mainwidget;
 }
 
@@ -851,8 +862,7 @@ void
 SoQt::mainLoop(void)
 {
   (void) qApp->exec();
-
-// Disabled invocation of SoQt::done(), since this calls SoDB::finish(), 
+// Disabled invocation of SoQt::done(), since this calls SoDB::finish(),
 // which means we would run into issues doing the usual...
 //    SoQt::mainLoop();
 //    root->unref();
@@ -873,7 +883,7 @@ SoQt::mainLoop(void)
 void
 SoQt::exitMainLoop(void)
 {
-  // FIXME: Should we call SoQt::done() here? 20060210 kyrah 
+  // FIXME: Should we call SoQt::done() here? 20060210 kyrah
   qApp->exit(0);
 }
 
@@ -883,6 +893,15 @@ SoQt::exitMainLoop(void)
 void
 SoQt::done(void)
 {
+#if defined(__COIN__) && defined(SOQT_SIGNAL_THREAD_ACTIVE)
+  SoQtP::signalthread->stopThread();
+  SoQtP::signalthread->wait();
+  delete SoQtP::signalthread;
+  SoQtP::signalthread = NULL;
+#endif //__COIN__ && SOQT_SIGNAL_THREAD_ACTIVE
+  delete SoQtP::imagereader;
+  SoQtP::imagereader = NULL;
+
   // To avoid getting any further invokations of
   // SoGuiP::sensorQueueChanged() (which would re-allocate the timers
   // we destruct below). This could for instance happen when
@@ -919,7 +938,7 @@ SoQt::done(void)
     }
   }
 
-#if 0 // FIXME: These methods exist in TGS Inventor. We should add 
+#if 0 // FIXME: These methods exist in TGS Inventor. We should add
       // them, and then call them from here. 20060210 kyrah
   SoInteraction::finish();
   SoNodeKit::finish();
