@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -11,13 +11,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -62,7 +62,7 @@
 
 static const char nullstring[] = "(null)";
 
-#define PRIVATE(obj) ((obj)->pimpl)
+#define PRIVATE(obj) (obj)
 #define PUBLIC(obj) ((obj)->pub)
 
 // *************************************************************************
@@ -74,7 +74,7 @@ static const char nullstring[] = "(null)";
 SbDict * SoQtComponentP::cursordict = NULL;
 
 SoQtComponentP::SoQtComponentP(SoQtComponent * o)
-  : SoGuiComponentP(o)
+  : SoGuiComponentP(o), classname(""), widgetname("")
 {
 }
 
@@ -100,11 +100,17 @@ SoQtComponentP::cleanupQtReferences(void)
   this->parent->removeEventFilter(this);
 }
 
+#if COIN_MAJOR_VERSION >= 3
+static void delete_dict_value(SbDict::Key key, void * value)
+{
+  delete (QCursor *)value;
+}
+#else // COIN_MAJOR_VERSION >= 3
 static void delete_dict_value(unsigned long key, void * value)
 {
   delete (QCursor *)value;
 }
-
+#endif // COIN_MAJOR_VERSION < 3
 void
 SoQtComponentP::atexit_cleanup()
 {
@@ -122,7 +128,7 @@ SoQtComponentP::getNativeCursor(const SoQtCursor::CustomCursor * cc)
 {
   if (SoQtComponentP::cursordict == NULL) { // first call, initialize
     SoQtComponentP::cursordict = new SbDict;
-    sogui_atexit((sogui_atexit_f*)SoQtComponentP::atexit_cleanup, 0);
+    SoAny::atexit((SoAny::atexit_f*)SoQtComponentP::atexit_cleanup, 0);
   }
 
   void * qc;
@@ -158,8 +164,13 @@ SoQtComponentP::getNativeCursor(const SoQtCursor::CustomCursor * cc)
   // Always 32x32 because that's what is recommended in the Qt
   // documentation for QCursor.  At least WinNT 4 will give us
   // "interesting" bugs for other cursor sizes.
+#if QT_VERSION >= 0x040000
+  QBitmap bitmap = QBitmap::fromData(QSize(32, 32), cursorbitmap, QImage::Format_MonoLSB);
+  QBitmap mask = QBitmap::fromData(QSize(32, 32), cursormask, QImage::Format_MonoLSB);
+#else
   QBitmap bitmap(32, 32, cursorbitmap, TRUE);
   QBitmap mask(32, 32, cursormask, TRUE);
+#endif
 
   // Sanity checks.
   assert(bitmap.size().width() > 0 && bitmap.size().height() > 0);
@@ -295,6 +306,12 @@ SoQtComponentP::eventFilter(QObject * obj, QEvent * e)
 
 #endif // DOXYGEN_SKIP_THIS
 
+#undef PUBLIC
+#undef PRIVATE
+
+#define PRIVATE(obj) ((obj)->pimpl)
+#define PUBLIC(obj) ((obj)->pub)
+
 // *************************************************************************
 
 SOQT_OBJECT_ABSTRACT_SOURCE(SoQtComponent);
@@ -335,21 +352,26 @@ SoQtComponent::SoQtComponent(QWidget * const parent,
   PRIVATE(this)->visibilitychangeCBs = NULL;
   PRIVATE(this)->fullscreen = FALSE;
 
-  if (name)
-    PRIVATE(this)->widgetname = name;
-
-  PRIVATE(this)->classname = "SoQtComponent";
+  this->setClassName("SoQtComponent");
 
   PRIVATE(this)->storesize.setValue(-1, -1);
-#ifdef Q_WS_MAC 
+#ifdef Q_WS_MAC
   PRIVATE(this)->windowsize.setValue(-1, -1);
 #endif
 
   SoAny::si()->addInternalFatalErrorHandler(SoQtComponentP::fatalerrorHandler,
                                             PRIVATE(this));
 
-  if ((parent == NULL) || ! embed) {
+  PRIVATE(this)->widgetname = (name ? name :
+			       this->getDefaultWidgetName());
+
+  if (!parent || !embed) {
+#if QT_VERSION >= 0x040000
+    PRIVATE(this)->parent = new QMainWindow();
+    PRIVATE(this)->parent->setObjectName(PRIVATE(this)->widgetname);
+#else
     PRIVATE(this)->parent = (QWidget *) new QMainWindow(NULL, name);
+#endif
     PRIVATE(this)->embedded = FALSE;
     PRIVATE(this)->shelled = TRUE;
   }
@@ -437,49 +459,65 @@ SoQtComponent::setClassName(const char * const name)
 void
 SoQtComponent::setBaseWidget(QWidget * widget)
 {
-//  SoDebugError::postInfo("SoQtComponent::setBaseWidget", "[invoked]");
+  QString iconText = this->getDefaultIconTitle();
+  QString widgetName = PRIVATE(this)->widgetname;
+
   assert(widget);
 
-//  if (PRIVATE(this)->parent)
-//    PRIVATE(this)->parent->removeEventFilter(PRIVATE(this));
-  if (PRIVATE(this)->widget)
+  if (PRIVATE(this)->widget) {
+#if QT_VERSION >= 0x040000
+    iconText = (PRIVATE(this)->widget->windowIconText().isEmpty() ?
+		iconText :
+		PRIVATE(this)->widget->windowIconText());
+    widgetName = (PRIVATE(this)->widget->objectName().isEmpty() ?
+		  widgetName :
+		  PRIVATE(this)->widget->objectName());
+#else
+    iconText = (PRIVATE(this)->widget->iconText().isEmpty() ?
+		iconText :
+		PRIVATE(this)->widget->iconText());
+    widgetName = (PRIVATE(this)->widget->name() == "" ?
+		  widgetName :
+		  QString(PRIVATE(this)->widget->name()));
+#endif
+
     PRIVATE(this)->widget->removeEventFilter(PRIVATE(this));
+    this->unregisterWidget(PRIVATE(this)->widget);
+  }
 
-  if (PRIVATE(this)->widget) { this->unregisterWidget(PRIVATE(this)->widget); }
   PRIVATE(this)->widget = widget;
   this->registerWidget(PRIVATE(this)->widget);
 
-//  PRIVATE(this)->parent = widget->parentWidget();
-
 #if 0 // debug
   SoDebugError::postInfo("SoQtComponent::setBaseWidget",
                          "widget: %p, parent: %p", w, PRIVATE(this)->parent);
 #endif // debug
 
-
   if (!PRIVATE(this)->parent || PRIVATE(this)->parent->isTopLevel()) {
-    if (PRIVATE(this)->captiontext.isNull()) PRIVATE(this)->captiontext = this->getDefaultTitle();
-    this->setTitle((const char *)PRIVATE(this)->captiontext.local8Bit());
+#if QT_VERSION >= 0x040000
+    if (PRIVATE(this)->widget->windowTitle() == "") {
+#else
+    if (PRIVATE(this)->widget->caption() == "") {
+#endif
+      this->setTitle(this->getDefaultTitle());
+    }
 
-    if (PRIVATE(this)->icontext.isNull()) PRIVATE(this)->icontext = this->getDefaultIconTitle();
-    SoQt::getShellWidget(this->getWidget())->setIconText(PRIVATE(this)->icontext);
+#if QT_VERSION >= 0x040000
+    SoQt::getShellWidget(this->getWidget())->setWindowIconText(iconText);
+#else
+    SoQt::getShellWidget(this->getWidget())->setIconText(iconText);
+#endif
   }
-
-  if (PRIVATE(this)->widgetname.isNull())
-    PRIVATE(this)->widgetname = this->getDefaultWidgetName();
-  PRIVATE(this)->widget->setName(PRIVATE(this)->widgetname);
-
+#if QT_VERSION >= 0x040000
+  PRIVATE(this)->widget->setObjectName(widgetName);
+#else
+  PRIVATE(this)->widget->setName(widgetName);
+#endif
   // Need this to auto-detect resize events.
-//  if (PRIVATE(this)->parent)
-//    PRIVATE(this)->parent->installEventFilter(PRIVATE(this));
-
   PRIVATE(this)->widget->installEventFilter(PRIVATE(this));
 
   QObject::connect(PRIVATE(this)->widget, SIGNAL(destroyed()),
                    PRIVATE(this), SLOT(widgetClosed()));
-
-//  if (storesize[0] != -1)
-//    PRIVATE(this)->widget->resize(QSize(storesize[0], storesize[1]));
 }
 
 // *************************************************************************
@@ -599,7 +637,7 @@ SbBool
 SoQtComponent::isVisible(void)
 {
   if (! PRIVATE(this)->widget) { return FALSE; }
-  return PRIVATE(this)->widget->isVisible() && PRIVATE(this)->widget->isVisibleToTLW();
+  return PRIVATE(this)->widget->isVisible();
 }
 
 // documented in common/SoGuiComponentCommon.cpp.in.
@@ -641,15 +679,17 @@ SoQtComponent::getParentWidget(void) const
 void
 SoQtComponent::setTitle(const char * const title)
 {
-  PRIVATE(this)->captiontext = title;
-
   if (this->getWidget()) {
     QWidget * toplevel = this->getWidget();
     while (!toplevel->isTopLevel() ) {
       toplevel = toplevel->parentWidget();
     }
     if (toplevel) {
+#if QT_VERSION >= 0x040000
+      toplevel->setWindowTitle(title);
+#else
       toplevel->setCaption(title);
+#endif
     }
   }
 }
@@ -658,19 +698,37 @@ SoQtComponent::setTitle(const char * const title)
 const char *
 SoQtComponent::getTitle(void) const
 {
-  return
-    PRIVATE(this)->captiontext.isNull() ? nullstring : (const char *) PRIVATE(this)->captiontext;
+  const char * result = "";
+
+  if (this->getWidget()) {
+    QWidget * toplevel = this->getWidget();
+    while (!toplevel->isTopLevel() ) {
+      toplevel = toplevel->parentWidget();
+    }
+    if (toplevel) {
+#if QT_VERSION >= 0x040000
+      result = toplevel->windowTitle().toUtf8().constData();
+#else
+      // Qt3 featured an implicit operator const char * () const
+      result = toplevel->caption();
+#endif
+    }
+  }
+
+  return result;
 }
 
 // documented in common/SoGuiComponentCommon.cpp.in.
 void
 SoQtComponent::setIconTitle(const char * const title)
 {
-  PRIVATE(this)->icontext = title;
-
   QWidget * w = this->getWidget();
   if (w && this->isTopLevelShell()) {
+#if QT_VERSION >= 0x040000
+    SoQt::getShellWidget(w)->setWindowIconText(title);
+#else
     SoQt::getShellWidget(w)->setIconText(title);
+#endif
   }
 }
 
@@ -678,22 +736,63 @@ SoQtComponent::setIconTitle(const char * const title)
 const char *
 SoQtComponent::getIconTitle(void) const
 {
-  return PRIVATE(this)->icontext.isNull() ? nullstring : (const char *)PRIVATE(this)->icontext;
+  const char * result = nullstring;
+
+  QWidget * w = this->getWidget();
+  if (w && this->isTopLevelShell()) {
+#if QT_VERSION >= 0x040000
+    QString iconText = SoQt::getShellWidget(w)->windowIconText();
+#else
+    QString iconText = SoQt::getShellWidget(w)->iconText();
+#endif
+
+    if (!iconText.isEmpty()) {
+#if QT_VERSION >= 0x040000
+      result = iconText.toUtf8().constData();
+#else
+    // Qt3 featured and implicit operator const char * () const
+      result = iconText;
+#endif
+    }
+  }
+
+  return result;
 }
 
 // documented in common/SoGuiComponentCommon.cpp.in.
 const char *
 SoQtComponent::getWidgetName(void) const
 {
-  return
-    PRIVATE(this)->widgetname.isNull() ? nullstring : (const char *)PRIVATE(this)->widgetname;
+  const char * result = nullstring;
+
+  if (!PRIVATE(this)->widgetname.isEmpty()) {
+#if QT_VERSION >= 0x040000
+    result = PRIVATE(this)->widgetname.toUtf8().constData();
+#else
+    // Qt3 featured an implicit operator const char * () const
+    result = PRIVATE(this)->widgetname;
+#endif
+  }
+
+  return result;
 }
 
 // documented in common/SoGuiComponentCommon.cpp.in.
 const char *
 SoQtComponent::getClassName(void) const
 {
-  return (const char *)PRIVATE(this)->classname;
+  const char * result = nullstring;
+
+  if (!PRIVATE(this)->classname.isEmpty()) {
+#if QT_VERSION >= 0x040000
+    result = PRIVATE(this)->classname.toUtf8().constData();
+#else
+    // Qt3 featured and implicit operator const char * () const
+    result = PRIVATE(this)->classname;
+#endif
+  }
+
+  return result;
 }
 
 // *************************************************************************
@@ -707,7 +806,7 @@ SoQtComponent::getClassName(void) const
 // regressions). The tests should at least cover these usage contexts
 // for SoQt component classes:
 //
-// 
+//
 //   * an SoQt-component viewer embedded in other Qt-widgets
 //   * a top-level (ie "free window") viewer
 //
@@ -814,7 +913,7 @@ SoQtComponent::setFullScreen(const SbBool onoff)
   // setWindowState() will preserve other window flags/states.
 #if HAVE_QWIDGET_SETWINDOWSTATE
   if (onoff) {
-#ifdef Q_WS_MAC 
+#ifdef Q_WS_MAC
     // Qt/Mac does not remember the window size when going fullscreen,
     // so when going back to windowed mode, the window will be 1x1 pixels
     // small -> we have to store the window size ourselves...
@@ -825,13 +924,13 @@ SoQtComponent::setFullScreen(const SbBool onoff)
     PRIVATE(this)->windowsize[1] = w->size().height();
 #endif
     w->setWindowState(w->windowState() | Qt::WindowFullScreen);
-#ifdef Q_WS_MAC 
+#ifdef Q_WS_MAC
     // Explicit show needed for Mac OS X, otherwise the window "vanishes"
     w->show();
 #endif
   } else {
     w->setWindowState(w->windowState() & ~Qt::WindowFullScreen);
-#ifdef Q_WS_MAC 
+#ifdef Q_WS_MAC
     w->resize(QSize(PRIVATE(this)->windowsize[0], PRIVATE(this)->windowsize[1]));
     w->show();
 #endif
@@ -846,7 +945,7 @@ SoQtComponent::setFullScreen(const SbBool onoff)
                             "QWidget::showFullScreen() method",
                             QT_VERSION_STR);
   return FALSE;
-#endif // !HAVE_QWIDGET_SHOWFULLSCREEN && !HAVE_QWIDGET_SETWINDOWSTATE 
+#endif // !HAVE_QWIDGET_SHOWFULLSCREEN && !HAVE_QWIDGET_SETWINDOWSTATE
   PRIVATE(this)->fullscreen = onoff;
   return TRUE;
 }
@@ -899,7 +998,7 @@ SoQtComponent::setWidgetCursor(QWidget * w, const SoQtCursor & cursor)
                                   "SOQT_NO_QTMAC_BUG_WARNINGS=1.\n");
       warningdisplayed = TRUE;
     }
-#else 
+#else
     const SoQtCursor::CustomCursor * cc = &cursor.getCustomCursor();
     w->setCursor(*SoQtComponentP::getNativeCursor(cc));
 #endif
@@ -907,19 +1006,19 @@ SoQtComponent::setWidgetCursor(QWidget * w, const SoQtCursor & cursor)
   else {
     switch (cursor.getShape()) {
     case SoQtCursor::DEFAULT:
-      w->setCursor(QCursor(Qt::arrowCursor));
+      w->setCursor(QCursor(Qt::ArrowCursor));
       break;
 
     case SoQtCursor::BUSY:
-      w->setCursor(QCursor(Qt::waitCursor));
+      w->setCursor(QCursor(Qt::WaitCursor));
       break;
 
     case SoQtCursor::CROSSHAIR:
-      w->setCursor(QCursor(Qt::crossCursor));
+      w->setCursor(QCursor(Qt::CrossCursor));
       break;
 
     case SoQtCursor::UPARROW:
-      w->setCursor(QCursor(Qt::upArrowCursor));
+      w->setCursor(QCursor(Qt::UpArrowCursor));
       break;
 
     default:
@@ -953,4 +1052,3 @@ SoQtComponent::setWidgetCursor(QWidget * w, const SoQtCursor & cursor)
 
 #undef PRIVATE
 #undef PUBLIC
-
