@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,13 +14,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -57,13 +57,13 @@ public:
   QWidget * parent;
   QWidget * widget;
   SbBool embedded, shelled;
-  QString classname, widgetname, captiontext, icontext;
+  QString classname, widgetname;
   SoQtComponentCB * closeCB;
   void * closeCBdata;
   SbPList * visibilitychangeCBs;
   SbBool realized;
   SbVec2s storesize;
-#ifdef Q_WS_MAC 
+#ifdef Q_WS_MAC
   // Workaround: Qt/Mac does not remember windowsize when going fullscreen
   SbVec2s windowsize;
 #endif
