@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -11,13 +11,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -38,8 +38,8 @@ enum SoQtCustomEvents {
   soqt6dofDevicePressureEvent
 };
 
-class SoQt6dofDevicePressureEvent : public QCustomEvent {
-  typedef QCustomEvent inherited;
+class SoQt6dofDevicePressureEvent : public QEvent {
+  typedef QEvent inherited;
 
 public:
   SoQt6dofDevicePressureEvent(void);
@@ -56,8 +56,8 @@ private:
 
 };
 
-class SoQt6dofDeviceButtonEvent : public QCustomEvent {
-  typedef QCustomEvent inherited;
+class SoQt6dofDeviceButtonEvent : public QEvent {
+  typedef QEvent inherited;
 
 public:
   SoQt6dofDeviceButtonEvent(void);
