@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -11,13 +11,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -366,14 +366,10 @@ SoQtKeyboard::translateEvent(QEvent * event)
     }
 #endif // Q_WS_X11
   }
-    
+
   SbBool keypress = event->type() == QEvent::KeyPress;
   SbBool keyrelease = event->type() == QEvent::KeyRelease;
 
-  // Qt 2 introduced "accelerator" type keyboard events.
-  keypress = keypress || (event->type() == QEvent::Accel);
-  keyrelease = keyrelease || (event->type() == QEvent::AccelAvailable);
-
   SbBool keyevent = keypress || keyrelease;
 
   if (keyevent && (PRIVATE(this)->eventmask & (KEY_PRESS|KEY_RELEASE))) {
@@ -389,8 +385,11 @@ SoQtKeyboard::translateEvent(QEvent * event)
     if (!PRIVATE(this)->kbdevent) PRIVATE(this)->kbdevent = new SoKeyboardEvent;
 
     // FIXME: check for Qt::Key_unknown. 19990212 mortene.
-
+#if QT_VERSION >= 0x040000
+    SbBool keypad = (keyevent->modifiers() & Qt::KeypadModifier) != 0;
+#else
     SbBool keypad = (keyevent->state() & QT_KEYPAD_MASK) != 0;
+#endif
 
     // Translate keycode Qt -> So
     void * table;
@@ -400,7 +399,7 @@ SoQtKeyboard::translateEvent(QEvent * event)
 #if 0 // disabled. Breaks build when compiling against OIV
       if (map->printable) PRIVATE(this)->kbdevent->setPrintableCharacter(map->printable);
 #endif // disabled
-    } 
+    }
     else if (!keypad && SoQtKeyboardP::translatetable->find(key, table)) {
       struct SoQtKeyboardP::key1map * map = (struct SoQtKeyboardP::key1map *)table;
       PRIVATE(this)->kbdevent->setKey(map->to);
@@ -421,39 +420,22 @@ SoQtKeyboard::translateEvent(QEvent * event)
     if (keyrelease) PRIVATE(this)->kbdevent->setState(SoButtonEvent::UP);
     else PRIVATE(this)->kbdevent->setState(SoButtonEvent::DOWN);
 
-    // Need to mask in or out modifiers to get the correct state, as
-    // the state() function gives us the situation immediately
-    // _before_ the event happened.
-    int state = keyevent->state();
-    if (keypress) {
-      switch (keyevent->key()) {
-      case Qt::Key_Shift: state |= Qt::ShiftButton; break;
-      case Qt::Key_Control: state |= Qt::ControlButton; break;
-      case Qt::Key_Alt: state |= Qt::AltButton; break;
-#if !(defined QT_ON_MAC && QT_VERSION >= 0x030100)
-// Since Qt/Mac 3.1.x, the "ctrl" key is mapped to Qt::MetaButton (not
-// Qt::ControlButton) => We must not handle META as ALT on Mac OS X.
-      case Qt::Key_Meta: state |= Qt::AltButton; break;
+#if QT_VERSION >= 0x040000
+    Qt::KeyboardModifiers state = keyevent->modifiers();
+#else
+    Qt::ButtonState state = keyevent->state();
 #endif
-      }
-    }
-    else {
-      switch (keyevent->key()) {
-      case Qt::Key_Shift: state &= ~Qt::ShiftButton; break;
-      case Qt::Key_Control: state &= ~Qt::ControlButton; break;
-      case Qt::Key_Alt: state &= ~Qt::AltButton; break;
-#if !(defined Q_WS_MAC && QT_VERSION >= 0x030100)
-// Since Qt/Mac 3.1.x, the "ctrl" key is mapped to Qt::MetaButton (not
-// Qt::ControlButton) => We must not handle META as ALT on Mac OS X.
-      case Qt::Key_Meta: state &= ~Qt::AltButton; break;
-#endif
-      }
-    }
 
     // Modifiers
+#if QT_VERSION >= 0x040000
+    PRIVATE(this)->kbdevent->setShiftDown(state & Qt::ShiftModifier);
+    PRIVATE(this)->kbdevent->setCtrlDown(state & Qt::ControlModifier);
+    PRIVATE(this)->kbdevent->setAltDown(state & Qt::AltModifier);
+#else
     PRIVATE(this)->kbdevent->setShiftDown(state & Qt::ShiftButton);
     PRIVATE(this)->kbdevent->setCtrlDown(state & Qt::ControlButton);
     PRIVATE(this)->kbdevent->setAltDown(state & Qt::AltButton);
+#endif
 
     // FIXME: read QCursor::position() instead,
     // and clean up this mess. 19990222 mortene.
@@ -475,4 +457,3 @@ SoQtKeyboard::translateEvent(QEvent * event)
 
 #undef PRIVATE
 #undef PUBLIC
-
