@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -11,13 +11,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -134,14 +134,27 @@ SoQtExaminerViewer::setCamera(SoCamera * newCamera)
 
   if (newCamera) {
     SoType camtype = newCamera->getTypeId();
-    SbBool orthotype =
+    SbBool orthogonal =
       camtype.isDerivedFrom(SoOrthographicCamera::getClassTypeId());
 
-    this->setRightWheelString(orthotype ? "Zoom" : "Dolly");
+    const char * oldLabel = this->getRightWheelString();
+    if (oldLabel) {
+      if (orthogonal) {
+        if (strcmp("Dolly",oldLabel) == 0)
+          this->setRightWheelString("Zoom");
+      }
+      else if (strcmp("Zoom",oldLabel) == 0)
+        this->setRightWheelString("Dolly");
+    }
     if (PRIVATE(this)->cameratogglebutton) {
-      PRIVATE(this)->cameratogglebutton->setPixmap(orthotype ?
-                                                   * (PRIVATE(this)->orthopixmap) :
-                                                   * (PRIVATE(this)->perspectivepixmap));
+#if QT_VERSION >= 0x040000
+      PRIVATE(this)->cameratogglebutton->setIcon(
+#else
+      PRIVATE(this)->cameratogglebutton->setPixmap(
+#endif
+	orthogonal ?
+	* (PRIVATE(this)->orthopixmap) :
+	* (PRIVATE(this)->perspectivepixmap));
     }
   }
 
@@ -180,7 +193,11 @@ SoQtExaminerViewer::createViewerButtons(QWidget * parent, SbPList * buttonlist)
     p = PRIVATE(this)->perspectivepixmap;
   else assert(0 && "unsupported cameratype");
 
+#if QT_VERSION >= 0x040000
+  PRIVATE(this)->cameratogglebutton->setIcon(*p);
+#else
   PRIVATE(this)->cameratogglebutton->setPixmap(*p);
+#endif
   PRIVATE(this)->cameratogglebutton->adjustSize();
 
   QObject::connect(PRIVATE(this)->cameratogglebutton, SIGNAL(clicked()),
@@ -246,4 +263,3 @@ SoQtExaminerViewerP::cameratoggleClicked()
 
 #undef PRIVATE
 #undef PUBLIC
-
