@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -11,13 +11,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -71,7 +71,7 @@
 #include <Inventor/Qt/widgets/SoQtThumbWheel.h>
 #include <Inventor/Qt/widgets/SoQtPopupMenu.h>
 #include <Inventor/Qt/viewers/SoQtFullViewer.h>
-#include <Inventor/Qt/SoAny.h> 
+#include <Inventor/Qt/SoAny.h>
 
 // Button icons.
 #include <Inventor/Qt/common/pixmaps/pick.xpm>
@@ -200,6 +200,10 @@ SoQtFullViewer::~SoQtFullViewer()
   delete [] this->rightWheelStr;
   delete [] this->bottomWheelStr;
 
+  if (this->prefmenu) {
+    delete this->prefmenu;
+  }
+
   delete PRIVATE(this);
 }
 
@@ -366,8 +370,13 @@ SoQtFullViewer::setViewing(SbBool enable)
   // Must check that buttons have been built, in case this viewer
   // component was made without decorations.
   if (PRIVATE(this)->viewerbuttons->getLength() > 0) {
+#if QT_VERSION >= 0x040000
+    PRIVATE(this)->getViewerbutton(EXAMINE_BUTTON)->setChecked(enable);
+    PRIVATE(this)->getViewerbutton(INTERACT_BUTTON)->setChecked(enable ? FALSE : TRUE);
+#else
     PRIVATE(this)->getViewerbutton(EXAMINE_BUTTON)->setOn(enable);
     PRIVATE(this)->getViewerbutton(INTERACT_BUTTON)->setOn(enable ? FALSE : TRUE);
+#endif
     PRIVATE(this)->getViewerbutton(SEEK_BUTTON)->setEnabled(enable);
   }
 }
@@ -394,16 +403,6 @@ SoQtFullViewer::buildWidget(QWidget * parent)
   PRIVATE(this)->viewerwidget->setBackgroundColor(QColor(250, 0, 0));
 #endif
 
-  // FIXME: The scaling and layout of a viewer without decorations
-  // currently don't work, so here I create the widget /with/
-  // decorations, and desides if they should be visible afterwards. A
-  // nice side-effect is that this makes sure that the
-  // (SoQtFullViewer*)->setDecoration(SbBool) works for TRUE and
-  // FALSE, regardless of how the widget initially was built. I regard
-  // this more like a workaround than as a /fix/, and lots and lots of
-  // this should have a major brushing as there still are a lot of
-  // evil-doers in this county.... 20021022 rolvs.
-
   PRIVATE(this)->canvas = inherited::buildWidget(PRIVATE(this)->viewerwidget);
 
   QSize s(PRIVATE(this)->viewerwidget->size().width(),
@@ -440,7 +439,10 @@ SoQtFullViewer::buildLeftTrim(QWidget * parent)
   QWidget * w = new QWidget(parent);
   w->setFixedWidth(30);
 
-  QGridLayout * gl = new QGridLayout(w, 3, 1, 2, -1);
+  QGridLayout * gl = new QGridLayout(w);
+#if QT_VERSION >= 0x040000
+  gl->setContentsMargins(0,0,0,0);
+#endif
   gl->addWidget(this->buildAppButtons(w), 0, 0);
 
   SoQtThumbWheel * t = new SoQtThumbWheel(SoQtThumbWheel::Vertical, w);
@@ -452,7 +454,7 @@ SoQtFullViewer::buildLeftTrim(QWidget * parent)
   QObject::connect(t, SIGNAL(wheelPressed()), PRIVATE(this), SLOT(leftWheelPressed()));
   QObject::connect(t, SIGNAL(wheelReleased()), PRIVATE(this), SLOT(leftWheelReleased()));
 
-  gl->addWidget(t, 2, 0, Qt::AlignBottom | Qt::AlignHCenter);
+  gl->addWidget(t, 1, 0, Qt::AlignBottom | Qt::AlignHCenter);
   gl->activate();
 
   return w;
@@ -498,20 +500,17 @@ SoQtFullViewer::buildBottomTrim(QWidget * parent)
 
   this->bottomWheelVal = t->value();
 
-  QGridLayout * gl = new QGridLayout(w, 1, 5);
-  gl->setColStretch(0, 0);
-  gl->setColStretch(1, 0);
-  gl->setColStretch(2, 0);
-  gl->setColStretch(3, 1);
-  gl->setColStretch(4, 0);
-
-  gl->addWidget(this->leftWheelLabel, 0, 0, Qt::AlignVCenter | Qt::AlignHCenter);
-  gl->addWidget(this->bottomWheelLabel, 0, 1, Qt::AlignVCenter | Qt::AlignRight);
-  gl->addWidget(t, 0, 2, Qt::AlignVCenter | Qt::AlignLeft);
-  gl->addWidget(this->rightWheelLabel, 0, 4, Qt::AlignVCenter | Qt::AlignRight);
-
-  gl->activate();
+  QGridLayout * layout = new QGridLayout(w);
+#if QT_VERSION >= 0x040000
+  layout->setContentsMargins(0,0,0,0);
+#endif
+  layout->addWidget(this->leftWheelLabel, 0, 0, Qt::AlignVCenter | Qt::AlignHCenter);
+  layout->addWidget(this->bottomWheelLabel, 0,1,  Qt::AlignVCenter | Qt::AlignRight);
+  layout->addWidget(t, 0,2, Qt::AlignVCenter | Qt::AlignLeft);
+  layout->addItem(new QSpacerItem(0,0,QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding),0, 3);
+  layout->addWidget(this->rightWheelLabel, 0, 4,Qt::AlignVCenter | Qt::AlignRight);
 
+  layout->activate();
   return w;
 }
 
@@ -539,7 +538,11 @@ SoQtFullViewer::buildRightTrim(QWidget * parent)
   QObject::connect(t, SIGNAL(wheelReleased()),
                    PRIVATE(this), SLOT(rightWheelReleased()));
 
-  QGridLayout * l = new QGridLayout(w, 3, 1, 2, -1);
+  QGridLayout * l = new QGridLayout(w);
+#if QT_VERSION >= 0x040000
+  l->setContentsMargins(0,0,0,0);
+  l->setVerticalSpacing(0);
+#endif
   l->setMargin(0);
   l->addWidget(this->buildViewerButtons(w), 0, 0);
   l->addWidget(t, 2, 0, Qt::AlignBottom | Qt::AlignHCenter);
@@ -555,6 +558,7 @@ QWidget *
 SoQtFullViewer::buildAppButtons(QWidget * parent)
 {
   PRIVATE(this)->appbuttonform = new QWidget(parent);
+  PRIVATE(this)->appbuttonform->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
 
   if (this->lengthAppPushButton() > 0)
     PRIVATE(this)->layoutAppButtons(PRIVATE(this)->appbuttonform);
@@ -575,9 +579,11 @@ SoQtFullViewer::buildViewerButtons(QWidget * parent)
 
   // assert(PRIVATE(this)->viewerbuttons->getLength() != 0);
   QGridLayout * l =
-    new QGridLayout(w, PRIVATE(this)->viewerbuttons->getLength(), 1);
+    new QGridLayout(w);
   l->setMargin(0);
-  l->setSpacing(0);
+#if QT_VERSION >= 0x040000
+  l->setVerticalSpacing(0);
+#endif
 
   const int numViewerButtons = PRIVATE(this)->viewerbuttons->getLength();
   for (int i = 0; i < numViewerButtons; i++) {
@@ -611,7 +617,7 @@ SoQtFullViewer::createViewerButtons(QWidget * parent, SbPList * buttonlist)
 #endif
 
 #if (defined Q_WS_MAC && QT_VERSION >= 0x030100) && defined(HAVE_QSTYLEFACTORY_H)
-    // Since Qt/Mac 3.1.x, all pushbuttons (even those < 32x32) are drawn 
+    // Since Qt/Mac 3.1.x, all pushbuttons (even those < 32x32) are drawn
     // using the Aqua style, i.e. with rounded edges and shading. This
     // looks really ugly in the viewer decoration. Drawing the buttons
     // in the Windows style gives us the flat, square buttons we want.
@@ -628,37 +634,65 @@ SoQtFullViewer::createViewerButtons(QWidget * parent, SbPList * buttonlist)
     switch (i) {
     case INTERACT_BUTTON:
       PRIVATE(this)->interactbutton = p;
+#if QT_VERSION >= 0x040000
+      p->setCheckable(TRUE);
+      p->setChecked(this->isViewing() ? FALSE : TRUE);
+      p->setIcon(QPixmap((const char **)pick_xpm));
+#else
       p->setToggleButton(TRUE);
-      p->setPixmap(QPixmap((const char **)pick_xpm));
       p->setOn(this->isViewing() ? FALSE : TRUE);
+      p->setPixmap(QPixmap((const char **)pick_xpm));
+#endif
       QObject::connect(p, SIGNAL(clicked()),
                        PRIVATE(this), SLOT(interactbuttonClicked()));
       break;
     case EXAMINE_BUTTON:
       PRIVATE(this)->viewbutton = p;
+#if QT_VERSION >= 0x040000
+      p->setCheckable(TRUE);
+      p->setChecked(this->isViewing());
+      p->setIcon(QPixmap((const char **)view_xpm));
+#else
       p->setToggleButton(TRUE);
-      p->setPixmap(QPixmap((const char **)view_xpm));
       p->setOn(this->isViewing());
+      p->setPixmap(QPixmap((const char **)view_xpm));
+#endif
       QObject::connect(p, SIGNAL(clicked()),
                        PRIVATE(this), SLOT(viewbuttonClicked()));
       break;
     case HOME_BUTTON:
       QObject::connect(p, SIGNAL(clicked()), PRIVATE(this), SLOT(homebuttonClicked()));
+#if QT_VERSION >= 0x040000
+      p->setIcon(QPixmap((const char **)home_xpm));
+#else
       p->setPixmap(QPixmap((const char **)home_xpm));
+#endif
       break;
     case SET_HOME_BUTTON:
       QObject::connect(p, SIGNAL(clicked()),
                        PRIVATE(this), SLOT(sethomebuttonClicked()));
+#if QT_VERSION >= 0x040000
+      p->setIcon(QPixmap((const char **)set_home_xpm));
+#else
       p->setPixmap(QPixmap((const char **)set_home_xpm));
+#endif
       break;
     case VIEW_ALL_BUTTON:
       QObject::connect(p, SIGNAL(clicked()),
                        PRIVATE(this), SLOT(viewallbuttonClicked()));
+#if QT_VERSION >= 0x040000
+      p->setIcon(QPixmap((const char **)view_all_xpm));
+#else
       p->setPixmap(QPixmap((const char **)view_all_xpm));
+#endif
       break;
     case SEEK_BUTTON:
       QObject::connect(p, SIGNAL(clicked()), PRIVATE(this), SLOT(seekbuttonClicked()));
+#if QT_VERSION >= 0x040000
+      p->setIcon(QPixmap((const char **)seek_xpm));
+#else
       p->setPixmap(QPixmap((const char **)seek_xpm));
+#endif
       break;
     default:
       assert(0);
@@ -727,6 +761,13 @@ SoQtFullViewer::setBottomWheelString(const char * const string)
 
 // *************************************************************************
 
+// Documented in common/viewers/SoGuiFullViewer.cpp.in.
+const char *
+SoQtFullViewer::getRightWheelString() const
+{
+  return this->rightWheelStr;
+}
+
 // Documented in common/viewers/SoGuiFullViewer.cpp.in.
 void
 SoQtFullViewer::setRightWheelString(const char * const string)
@@ -769,7 +810,7 @@ SoQtFullViewer::sizeChanged(const SbVec2s & size)
 #if defined Q_WS_MAC && ((QT_VERSION == 0x030100) || (QT_VERSION == 0x030101))
 
   // Environment variable to override Qt/Mac 3.1.x workarounds.
-  const char * forcenoresizeworkaround = 
+  const char * forcenoresizeworkaround =
     SoAny::si()->getenv("FORCE_NO_QTMAC_31_RESIZE_WORKAROUND");
   if (!forcenoresizeworkaround || (atoi(forcenoresizeworkaround) == 0)) {
 
@@ -778,10 +819,10 @@ SoQtFullViewer::sizeChanged(const SbVec2s & size)
 
       // spit out a warning that this is a Qt/Mac bug, not an SoQt problem
       const char * env = SoAny::si()->getenv("SOQT_NO_QTMAC_BUG_WARNINGS");
-      if (!env || !atoi(env)) {        
-        SoDebugError::postWarning("SoQtFullViewer::sizeChanged", 
+      if (!env || !atoi(env)) {
+        SoDebugError::postWarning("SoQtFullViewer::sizeChanged",
                                   "\nThis version of Qt/Mac contains a bug "
-                                  "that makes it necessary to leave the\n"  
+                                  "that makes it necessary to leave the\n"
                                   "lowermost 15 pixels of the viewer window "
                                   "blank. Set the environment variable\n"
                                   "FORCE_NO_QTMAC_31_RESIZE_WORKAROUND=1 to "
@@ -789,11 +830,11 @@ SoQtFullViewer::sizeChanged(const SbVec2s & size)
                                   "You can turn off warnings about Qt/Mac "
                                   "bugs permanently by setting \n"
                                   "SOQT_NO_QTMAC_BUG_WARNINGS=1.\n");
-      } 
-    } 
+      }
+    }
   }
 
-#endif 
+#endif
 
   newsize = SbVec2s(SoQtMax(newsize[0], (short)1),
                     SoQtMax(newsize[1], (short)1));
@@ -817,7 +858,6 @@ SoQtFullViewerP::showDecorationWidgets(SbBool onOff)
   if (this->mainlayout) delete this->mainlayout;
 
   assert(this->viewerwidget);
-//  assert(this->canvasparent);
 
   assert(PUBLIC(this)->leftDecoration && PUBLIC(this)->bottomDecoration && PUBLIC(this)->rightDecoration);
   if (onOff) {
@@ -825,24 +865,39 @@ SoQtFullViewerP::showDecorationWidgets(SbBool onOff)
     PUBLIC(this)->bottomDecoration->show();
     PUBLIC(this)->rightDecoration->show();
 
-    QGridLayout * g = new QGridLayout(this->viewerwidget, 2, 1, 0, -1); // VIEWERBORDER);
+#if QT_VERSION >= 0x040000
+    this->viewerwidget->setContentsMargins(0,0,0,0);
+#endif
+
+    QGridLayout * g = new QGridLayout(this->viewerwidget); // VIEWERBORDER);
+    g->setSpacing(0);
+#if QT_VERSION >= 0x040000
+    g->setContentsMargins(0,0,0,0);
+#endif
 
     g->addWidget(PUBLIC(this)->bottomDecoration, 1, 0);
 
-    QGridLayout * subLayout = new QGridLayout(1, 3, 0);
+    QGridLayout * subLayout = new QGridLayout();
     g->addLayout(subLayout, 0, 0);
+#if QT_VERSION >= 0x040000
+    subLayout->setVerticalSpacing(0);
+    subLayout->setContentsMargins(0,0,0,0);
+#endif
+
+    PUBLIC(this)->leftDecoration->setSizePolicy(QSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum));
 
     subLayout->addWidget(PUBLIC(this)->leftDecoration, 0, 0);
+    this->canvas->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
     subLayout->addWidget(this->canvas, 0, 1);
     subLayout->addWidget(PUBLIC(this)->rightDecoration, 0, 2);
 
-//     subLayout->setColStretch(1, 1);
-//     g->setRowStretch(0, 1);
-
     this->mainlayout = g;
   } else {
-    QGridLayout * g = new QGridLayout(this->viewerwidget, 1, 1, 0, -1);
+    QGridLayout * g = new QGridLayout(this->viewerwidget);
     g->addWidget(this->canvas, 0, 0);
+#if QT_VERSION >= 0x040000
+    g->setContentsMargins(0,0,0,0);
+#endif
     this->mainlayout = g;
 
     PUBLIC(this)->leftDecoration->hide();
@@ -867,12 +922,16 @@ SoQtFullViewerP::layoutAppButtons(QWidget * form)
   int nrbuttons = this->appbuttonlist->getLength();
   if (nrbuttons == 0) return;
 
-  this->appbuttonlayout = new QGridLayout(form, nrbuttons, 1);
+  this->appbuttonlayout = new QGridLayout(form);
+#if QT_VERSION >= 0x040000
+  this->appbuttonlayout->setContentsMargins(0,0,0,0);
+#endif
+  this->appbuttonlayout->setSpacing(0);
 
   for (int i=0; i < nrbuttons; i++) {
     QWidget * button = (QWidget *)((*(this->appbuttonlist))[i]);
-    button->show();
     ((QGridLayout *)this->appbuttonlayout)->addWidget(button, i, 0);
+    button->show();
   }
 
   this->appbuttonlayout->activate();
@@ -885,9 +944,17 @@ void
 SoQtFullViewerP::interactbuttonClicked(void)
 {
   if (this->interactbutton)
+#if QT_VERSION >= 0x040000
+    ((QPushButton *)this->interactbutton)->setChecked(TRUE);
+#else
     ((QPushButton *)this->interactbutton)->setOn(TRUE);
+#endif
   if (this->viewbutton)
+#if QT_VERSION >= 0x040000
+    ((QPushButton *)this->viewbutton)->setChecked(FALSE);
+#else
     ((QPushButton *)this->viewbutton)->setOn(FALSE);
+#endif
   if (PUBLIC(this)->isViewing())
     PUBLIC(this)->setViewing(FALSE);
 }
@@ -899,9 +966,17 @@ void
 SoQtFullViewerP::viewbuttonClicked(void)
 {
   if (this->interactbutton)
+#if QT_VERSION >= 0x040000
+    ((QPushButton *)this->interactbutton)->setChecked(FALSE);
+#else
     ((QPushButton *)this->interactbutton)->setOn(FALSE);
+#endif
   if (this->viewbutton)
+#if QT_VERSION >= 0x040000
+    ((QPushButton *)this->viewbutton)->setChecked(TRUE);
+#else
     ((QPushButton *)this->viewbutton)->setOn(TRUE);
+#endif
   if (!PUBLIC(this)->isViewing())
     PUBLIC(this)->setViewing(TRUE);
 }
@@ -1023,4 +1098,3 @@ void SoQtFullViewerP::rightWheelReleased(void) { PUBLIC(this)->rightWheelFinish(
 
 #undef PUBLIC
 #undef PRIVATE
-
