@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -11,13 +11,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -29,9 +29,8 @@
 
 // *************************************************************************
 
-// FIXME: get rid of this define. We should fix up the compile issues
-// wrt Qt 4 properly. 20050629 mortene.
-#define QT3_SUPPORT
+// FIXME: create a new Qt4NativePopupMenu. There are just too
+// many differences between Qt3 menu handling and Qt4 menu handling.
 
 #include <qmetaobject.h>
 
@@ -40,6 +39,7 @@
 // Take care of class name incompatibilities between Qt 3 and Qt 4.
 
 #if QT_VERSION < 0x040000 // pre Qt 4
+#include <qaction.h>
 #include <qpopupmenu.h>
 #define QPOPUPMENU_CLASS QPopupMenu
 #else // Qt 4.0.0+
@@ -63,6 +63,7 @@ struct MenuRecord {
   int menuid;
   char * name;
   char * title;
+  QAction * action;
   QPOPUPMENU_CLASS * menu;
   QPOPUPMENU_CLASS * parent;
 }; // struct MenuRecord
@@ -72,6 +73,7 @@ struct ItemRecord {
   int flags;
   char * name;
   char * title;
+  QAction * action;
   QPOPUPMENU_CLASS * parent;
 }; // struct ItemRecord
 
@@ -121,6 +123,10 @@ QtNativePopupMenu::~QtNativePopupMenu()
     delete [] rec->title;
     delete rec;
   }
+
+  delete this->menus;
+  delete this->items;
+
 } // ~QtNativePopupMenu()
 
 // *************************************************************************
@@ -183,8 +189,10 @@ QtNativePopupMenu::setMenuTitle(
   delete [] rec->title;
   rec->title = strcpy(new char [strlen(title)+1], title);
 #if QT_VERSION >= 200
-  if (rec->parent)
-    rec->parent->changeItem(rec->menuid, QString(rec->title));
+  if (rec->parent) {
+    //rec->parent->actionAt(QPoint(rec->menuid,0))->setText(rec->title);
+    getMenuRecord(rec->menuid)->action->setText(rec->title);
+  }
 #else // Qt version < 2.0.0
   // This QMenuData::changeItem() method is being obsoleted from Qt
   // from version 2.0.0 onwards.
@@ -260,7 +268,11 @@ QtNativePopupMenu::setMenuItemTitle(
   rec->title = strcpy(new char [strlen(title)+1], title);
 #if QT_VERSION >= 200
   if (rec->parent)
+#if QT_VERSION >= 0x040000
+    rec->parent->actionAt(QPoint(rec->itemid,0))->setText(rec->title);
+#else
     rec->parent->changeItem(rec->itemid, QString(rec->title));
+#endif
 #else // Qt version < 2.0.0
   // This QMenuData::changeItem() method is being obsoleted from Qt
   // from version 2.0.0 onwards.
@@ -290,29 +302,49 @@ QtNativePopupMenu::setMenuItemEnabled(int itemid,
 {
   ItemRecord * rec = this->getItemRecord(itemid);
   if (rec) {
+#if QT_VERSION >= 0x040000
+    rec->parent->actionAt(QPoint(rec->itemid,0))->setEnabled(enabled ? true : false);
+#else
     rec->parent->setItemEnabled(rec->itemid, enabled ? true : false);
+#endif
     return;
   }
   MenuRecord * mrec = this->getMenuRecord(itemid);
   assert(mrec && "no such menu");
   assert(mrec->parent && "a menuitem must have a parent to be enabled/disabled");
+
+#if QT_VERSION >= 0x040000
+  mrec->parent->actionAt(QPoint(mrec->menuid,0))->setEnabled(enabled ? true : false);
+#else
   mrec->parent->setItemEnabled(mrec->menuid, enabled ? true : false);
+#endif
 } // setMenuItemEnabled()
 
 /*!
 */
 
 SbBool
-QtNativePopupMenu::getMenuItemEnabled(int itemid) 
+QtNativePopupMenu::getMenuItemEnabled(int itemid)
 {
   ItemRecord * rec = this->getItemRecord(itemid);
-  if (rec) return rec->parent->isItemEnabled(rec->itemid) ? TRUE : FALSE;
+
+  if (rec) {
+#if QT_VERSION >= 0x040000
+    return rec->parent->actionAt(QPoint(rec->itemid,0))->isEnabled();
+#else
+    return rec->parent->isItemEnabled(rec->itemid) ? TRUE : FALSE;
+#endif
+  }
 
   MenuRecord * mrec = this->getMenuRecord(itemid);
   assert(mrec && "no such menu");
   assert(mrec->parent && "a menuitem must have a parent to be enabled/disabled");
-  
+
+#if QT_VERSION >= 0x040000
+  return mrec->parent->actionAt(QPoint(mrec->menuid,0))->isEnabled();
+#else
   return mrec->parent->isItemEnabled(mrec->menuid) ? TRUE : FALSE;
+#endif
 } // getMenuItemEnabled()
 
 /*!
@@ -328,11 +360,15 @@ QtNativePopupMenu::_setMenuItemMarked(int itemid, SbBool marked)
     rec->flags |= ITEM_MARKED;
   else
     rec->flags &= ~ITEM_MARKED;
+
   if (rec->parent != NULL) {
-#if QT_VERSION >= 0x040000 // Qt 4.*
-    // FIXME: is this really safe? (20050727 frodo)
-    QAction * action = (QAction *) rec->parent->findItem(itemid);
-    if (action) action->setChecked(marked ? true : false);
+#if QT_VERSION >= 0x040000
+    QAction * action = rec->action;
+    if (action) {
+      //FIXME: Could this be done on creation? - BFG 20090910
+      action->setCheckable(true);
+      action->setChecked(marked ? true : false);
+    }
 #else
     rec->parent->setItemChecked(rec->itemid, marked ? true : false);
 #endif
@@ -350,7 +386,14 @@ QtNativePopupMenu::getMenuItemMarked(
   assert(rec && "no such menu");
   if (rec->parent == NULL)
     return (rec->flags & ITEM_MARKED) ? TRUE : FALSE;
+
+#if QT_VERSION >= 0x040000
+  QAction * action = rec->action;
+  if (!action) return false;
+  return action->isChecked();
+#else
   return rec->parent->isItemChecked(rec->itemid) ? TRUE : FALSE;
+#endif
 } // getMenuItemMarked()
 
 // *************************************************************************
@@ -373,16 +416,32 @@ QtNativePopupMenu::addMenu(int menuid,
   // itemActivation(). Multiple calls to itemActivation() causes a
   // segfault when selecting Quadbuffer stereo, at least when it's not
   // supported. (20050726 frodo)
-#if QT_VERSION >= 0x040000 // Qt 4.*
+
+  // this was changed/fixed again in Qt 4.4.0, so now we shouldn't
+  // disconnect menu items if this version is detected...
+  // (20070530 pederb)
+#if (QT_VERSION >= 0x040000) && (QT_VERSION < 0x040400)
   QObject::disconnect(sub->menu, SIGNAL(activated(int)),
                       this, SLOT(itemActivation(int)));
-#endif
+#endif // QT-version >= 400 && QT-version < 4.4.0
 
+#if QT_VERSION >= 0x040000
+  QAction * action;
+  if (pos == -1) {
+    action = super->menu->addMenu(sub->menu);
+  }
+  else {
+    QAction * before = super->menu->actionAt(QPoint(pos,0));
+    action = super->menu->insertMenu(before,sub->menu);
+  }
+  action->setText(QString(sub->title));
+#else
   if (pos == -1)
     super->menu->insertItem(QString(sub->title), sub->menu, sub->menuid);
-  else 
+  else
     super->menu->insertItem(QString(sub->title),
-                             sub->menu, sub->menuid, pos);
+			    sub->menu, sub->menuid, pos);
+#endif
   sub->parent = super->menu;
 } // addMenu()
 
@@ -399,20 +458,31 @@ QtNativePopupMenu::addMenuItem(int menuid,
   ItemRecord * item = this->getItemRecord(itemid);
   assert(item && "invalid child menu id");
 
+#if QT_VERSION >= 0x040000
+  item->action = new QAction(menu->menu);
+  item->action->setText(item->title);
+  if (pos == -1) {
+    menu->menu->addAction(item->action);
+  }
+  else {
+    menu->menu->insertAction(menu->menu->actionAt(QPoint(pos,0)),item->action);
+  }
+#else
   if (pos == -1)
     menu->menu->insertItem(QString(item->title), item->itemid);
   else
     menu->menu->insertItem(QString(item->title), item->itemid, pos);
+#endif
   item->parent = menu->menu;
 
-#if QT_VERSION >= 0x040000 // Qt 4.*
+#if QT_VERSION >= 0x040000
   // FIXME: is this really safe? (20050726 frodo)
-  QAction * action = (QAction *) item->parent->findItem(itemid);
+  QAction * action = (QAction *) item->parent->actionAt(QPoint(itemid,0));
   if (action) action->setCheckable(true);
 #endif // Qt 4.*
-  
+
   if (item->flags & ITEM_MARKED) {
-#if QT_VERSION >= 0x040000 // Qt 4.*
+#if QT_VERSION >= 0x040000
     if (action) action->setChecked(true);
 #else
     item->parent->setItemChecked(item->itemid, true);
@@ -426,9 +496,13 @@ QtNativePopupMenu::addSeparator(int menuid,
 {
   MenuRecord * menu = this->getMenuRecord(menuid);
   assert(menu && "no such menu");
-  
+
   ItemRecord * rec = createItemRecord("separator");
+#if QT_VERSION >= 0x040000
+  menu->menu->insertSeparator(menu->menu->actionAt(QPoint(pos,0)));
+#else
   menu->menu->insertSeparator(pos);
+#endif
   rec->flags |= ITEM_SEPARATOR;
   this->items->append(rec);
 } // addSeparator()
@@ -458,7 +532,11 @@ QtNativePopupMenu::removeMenu(int menuid)
 #endif // SOQT_DEBUG
     return;
   }
+#if QT_VERSION >= 0x040000
+  rec->parent->removeAction(rec->parent->actionAt(QPoint(rec->menuid,0)));
+#else
   rec->parent->removeItem(rec->menuid);
+#endif
   rec->parent = NULL;
 } // removeMenu()
 
@@ -481,7 +559,11 @@ QtNativePopupMenu::removeMenuItem(int itemid)
 #endif // SOQT_DEBUG
     return;
   }
+#if QT_VERSION >= 0x040000
+  rec->parent->removeAction(rec->parent->actionAt(QPoint(rec->itemid,0)));
+#else
   rec->parent->removeItem(rec->itemid);
+#endif
   rec->parent = NULL;
 } // removeMenuItem()
 
@@ -495,7 +577,7 @@ QtNativePopupMenu::popUp(QWidget * inside, int x, int y)
 
   // Use exec() and not popup(). popup() doesn't seem to work properly
   // with a GL widget (the GL-redraw seems to overwrite the popup
-  // widget or something). pederb, 2003-10-01 
+  // widget or something). pederb, 2003-10-01
   //
   // An additional piece of information: the popup-menu misbehavior is
   // that at first attempt just a "shadow" comes up, and then on the
@@ -542,6 +624,22 @@ QtNativePopupMenu::getItemRecord(int itemid)
 
 // *************************************************************************
 
+/*!
+*/
+ItemRecord *
+QtNativePopupMenu::getItemRecordFromAction(QAction * action)
+{
+  const int numItems = this->items->getLength();
+  for (int i = 0; i < numItems; i++) {
+    const ItemRecord * rec = static_cast<ItemRecord *>((*this->items)[i]);
+    if (rec->action == action) { return (ItemRecord *) (*this->items)[i]; }
+  }
+
+  return (ItemRecord *) NULL;
+}
+
+// *************************************************************************
+
 /*!
 */
 
@@ -554,16 +652,18 @@ QtNativePopupMenu::createMenuRecord(
   rec->name = strcpy(new char [strlen(name)+1], name);
   rec->title = strcpy(new char [strlen(name)+1], name);
 
-#if QT_VERSION >= 0x040000 // Qt 4.*
+#if QT_VERSION >= 0x040000
   rec->menu = new QPOPUPMENU_CLASS(QString(name));
+  QObject::connect(rec->menu, SIGNAL(triggered(QAction *)),
+                   this, SLOT(itemActivation(QAction *)));
 #else
   rec->menu = new QPOPUPMENU_CLASS((QWidget *) NULL, name);
-#endif
-
   QObject::connect(rec->menu, SIGNAL(activated(int)),
                    this, SLOT(itemActivation(int)));
+#endif
 
   rec->parent = NULL;
+  rec->action = NULL;
   return rec;
 } // create()
 
@@ -580,6 +680,7 @@ QtNativePopupMenu::createItemRecord(
   rec->name = strcpy(new char [strlen(name)+1], name);
   rec->title = strcpy(new char [strlen(name)+1], name);
   rec->parent = NULL;
+  rec->action = NULL;
   return rec;
 } // create()
 
@@ -587,9 +688,20 @@ QtNativePopupMenu::createItemRecord(
 
 void
 QtNativePopupMenu::itemActivation(// private slot
-  int itemid)
+  QAction * action
+  )
 {
-  inherited::invokeMenuSelection(itemid);
+  ItemRecord * rec = getItemRecordFromAction(action);
+  assert(rec);
+  inherited::invokeMenuSelection(rec->itemid);
 } // menuSelection()
 
+void
+QtNativePopupMenu::itemActivation(// private slot used for Qt3 code path
+  int itemid
+  )
+{
+  inherited::invokeMenuSelection(itemid);
+}
+
 // *************************************************************************
