@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) by Kongsberg Oil & Gas Technologies.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -11,13 +11,13 @@
  *
  *  For using Coin with software that can not be combined with the GNU
  *  GPL, and for taking advantage of the additional benefits of our
- *  support services, please contact Systems in Motion about acquiring
- *  a Coin Professional Edition License.
+ *  support services, please contact Kongsberg Oil & Gas Technologies
+ *  about acquiring a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
- *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  Kongsberg Oil & Gas Technologies, Bygdoy Alle 5, 0257 Oslo, NORWAY.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -79,11 +79,17 @@ SoQtGLArea::SoQtGLArea(QGLFormat * const format,
                        QWidget * const parent,
                        const QGLWidget * sharewidget,
                        const char * const name)
-  // The 3rd argument is supposed to be the widget name, but when
-  // running on QGL v4.30 and Qt v2.1.0 application code will crash on
-  // exit under freak conditions.
-  : inherited(*format, parent, NULL, sharewidget, Qt::WResizeNoErase)
+#if QT_VERSION >= 0x040000
+   : inherited(*format, parent, sharewidget)
+#else
+   : inherited(*format, parent, NULL, sharewidget, Qt::WResizeNoErase)
+#endif
 {
+#if QT_VERSION >= 0x040000
+  this->setObjectName(name);
+#else
+  this->setName(name);
+#endif
 #if HAVE_QGLWIDGET_SETAUTOBUFFERSWAP
   // We'll handle the OpenGL buffer swapping ourselves, to support the
   // different combinations of rendering options (doublebuffer with
@@ -105,7 +111,6 @@ void
 SoQtGLArea::initializeGL(void)
 {
   SOQT_GLAREA_DEBUG_START(initializeGL);
-  this->setBackgroundMode(Qt::NoBackground); // Avoid unnecessary flicker.
   emit this->init_sig();
   SOQT_GLAREA_DEBUG_DONE(initializeGL);
 }
@@ -137,10 +142,10 @@ SoQtGLArea::focusInEvent(QFocusEvent * e)
   // Here's what the QWidget implementation of this method does:
 
 //     if ( focusPolicy() != NoFocus || !isTopLevel() ) {
-// 	update();
-// 	if ( testWState(WState_AutoMask) )
-// 	    updateMask();
-// 	setMicroFocusHint(width()/2, 0, 1, height(), FALSE);
+//        update();
+//        if ( testWState(WState_AutoMask) )
+//            updateMask();
+//        setMicroFocusHint(width()/2, 0, 1, height(), FALSE);
 //     }
 
   // QWidget::update() calls repaint(), which causes paintGL() to be
@@ -174,7 +179,7 @@ SoQtGLArea::event(QEvent * e)
   // SoQtExaminerViewer, use of the mousewheel over a viewer that
   // doesn't have the focus causes wheel events to end up in both the
   // viewer under the mouse and the viewer that has the focus.
-  // 
+  //
   // Our workaround is thus to ignore a wheel event when the widget
   // doesn't have the focus.
   //
